package com.hisschemoller.model 
{
	import com.hisschemoller.model.vo.FLARResultVO;
	
	import org.libspark.flartoolkit.core.FLARCode;
	import org.libspark.flartoolkit.core.param.FLARParam;
	import org.libspark.flartoolkit.core.raster.rgb.FLARRgbRaster_BitmapData;
	import org.libspark.flartoolkit.detector.FLARSingleMarkerDetector;
	import org.puremvc.as3.interfaces.IProxy;
	import org.puremvc.as3.patterns.proxy.Proxy;
	
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.PixelSnapping;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.SecurityErrorEvent;
	import flash.events.TimerEvent;
	import flash.media.Camera;
	import flash.media.Video;
	import flash.net.URLLoader;
	import flash.net.URLLoaderDataFormat;
	import flash.net.URLRequest;
	import flash.utils.Timer;	
	/**
	 * @author Wouter Hisschemöller
	 * (c) Jul 30, 2009
	 */
	public class FLARProxy extends Proxy implements IProxy 
	{
		public static const NAME : String = "FLARProxy";
		public static const START : String = "startFLAR";
		public static const TRANSFORM_3D_UPDATE : String = "transform3DUpdate";
		private var mLoader : URLLoader;
		private var mParam : FLARParam;
		private var mCode : FLARCode;
		private var mRaster : FLARRgbRaster_BitmapData; 
		private var mDetector : FLARSingleMarkerDetector;
		private var mResultVO : FLARResultVO;
		private var mCamera : Camera;
		private var mVideo : Video;
		private var mCaptureBitmap : Bitmap;
		private var mTimer : Timer;
		private var mCameraFile : String = "../bin/camera_para.dat";
		private var mCodeFile : String = "../bin/flarlogo.pat";
		private var mWidth : int = 320;
		private var mHeight : int = 240;
		private var mHalfWidth : int = mWidth * 0.5;
		private var mHalfHeight : int = mHeight * 0.5;
		private var mFPS : int = 25;
		private var mCodeWidth : int = 80;

		public function FLARProxy()
		{
			super(NAME, null);
		}

		/**
		 * 
		 */
		public function getParam() : FLARParam
		{
			return mParam;
		}

		/**
		 * 
		 */
		public function getBitmap() : Bitmap
		{
			return mCaptureBitmap;
		}

		/**
		 * 
		 */
		public function getWidth() : int
		{
			return mWidth;
		}

		/**
		 * 
		 */
		public function getHeight() : int
		{
			return mHeight;
		}

		/**
		 * 
		 */
		override public function onRegister() : void
		{
			mLoader = new URLLoader();
			mLoader.dataFormat = URLLoaderDataFormat.BINARY;
			mLoader.addEventListener(Event.COMPLETE, handleParamLoaded);
			mLoader.addEventListener(IOErrorEvent.IO_ERROR, handleIOErrorEvent);
			mLoader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, handleSecurityErrorEvent);
			mLoader.load(new URLRequest(mCameraFile));
		}

		/**
		 * 
		 */
		private function handleParamLoaded(inEvent : Event) : void 
		{
			mLoader.removeEventListener(Event.COMPLETE, handleParamLoaded);
			mParam = new FLARParam();
			mParam.loadARParam(mLoader.data);
			mParam.changeScreenSize(mWidth, mHeight);
			
			mLoader.dataFormat = URLLoaderDataFormat.TEXT;
			mLoader.addEventListener(Event.COMPLETE, handleCodeLoaded);
			mLoader.load(new URLRequest(mCodeFile));
		}

		/**
		 * 
		 */
		private function handleCodeLoaded(inEvent : Event) : void 
		{
			mCode = new FLARCode(16, 16);
			mCode.loadARPatt(mLoader.data);
			
			mLoader.removeEventListener(Event.COMPLETE, handleCodeLoaded);
			mLoader.removeEventListener(IOErrorEvent.IO_ERROR, handleIOErrorEvent);
			mLoader.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, handleSecurityErrorEvent);
			mLoader = null;
			
			start();
		}

		/**
		 * 
		 */
		private function handleIOErrorEvent(inEvent : IOErrorEvent) : void 
		{
			throw new Error("IOError : " + inEvent.text);
		}

		/**
		 * 
		 */
		private function handleSecurityErrorEvent(inEvent : SecurityErrorEvent) : void 
		{
			throw new Error("SecurityError : " + inEvent.text);
		}

		/**
		 * 
		 */
		private function start() : void 
		{
			// setup webcam
			mCamera = Camera.getCamera();
			
			if(!mCamera) throw new Error('No webcam!!!!');
			
			mCamera.setMode(mWidth, mHeight, mFPS);
			mVideo = new Video(mWidth, mHeight);
			mVideo.attachCamera(mCamera);
			mCaptureBitmap = new Bitmap(new BitmapData(mWidth, mHeight, false, 0), PixelSnapping.AUTO, true);
			
			// setup ARToolkit
			mRaster = new FLARRgbRaster_BitmapData(mCaptureBitmap.bitmapData);
			mDetector = new FLARSingleMarkerDetector(mParam, mCode, mCodeWidth);
			
			// setup data object
			mResultVO = new FLARResultVO();
			
			// setup timer
			mTimer = new Timer(int(1000 / mFPS));
			mTimer.addEventListener(TimerEvent.TIMER, handleTimerEvent);
			mTimer.start();
			
			sendNotification(START);
		}

		/**
		 * 
		 */
		private function handleTimerEvent(inEvent : TimerEvent) : void 
		{
			mCaptureBitmap.bitmapData.draw(mVideo);
			
			if(mDetector.detectMarkerLite(mRaster, 80) && mDetector.getConfidence() > 0.5) 
			{
				mDetector.getTransformMatrix(mResultVO.resultMat);
				
				mResultVO.square = mDetector.getSquare();
				mResultVO.relativeX = (mResultVO.square.label.pos_x - mHalfWidth) / mHalfWidth;
				mResultVO.relativeY = (mResultVO.square.label.pos_y - mHalfHeight) / mHalfHeight;
				
				sendNotification(TRANSFORM_3D_UPDATE, mResultVO);
			}
			else
			{
				sendNotification(TRANSFORM_3D_UPDATE, null);
			}
		}
	}
}
