package com.hisschemoller.view.components.audio 
{
	import flash.events.SampleDataEvent;
	import flash.media.Sound;
	import flash.media.SoundChannel;
	import flash.utils.getTimer;
	import flash.utils.setTimeout;	
	/**
	 * @author Wouter Hisschemöller
	 * (c) Aug 23, 2009
	 */
	public class AudioDriver 
	{
		public static const BUFFER_SIZE : int = 3072;
		public static const SAMPLE_RATE : int = 44100;
		private const BUFFER : Vector.<Vector.<Number>> = new Vector.<Vector.<Number>>(2, true);
		private const SOUND : Sound = new Sound();
		private var mSoundChannel : SoundChannel;
		private var mSoundLatency : Number = 0;
		private var mLastBuffer : int;
		private var mEngine : AudioEngine;

		public function AudioDriver(inEngine : AudioEngine)
		{
			mEngine = inEngine;
			
			// Stereo (2 channels)
			BUFFER[0] = new Vector.<Number>(BUFFER_SIZE, true);
			BUFFER[1] = new Vector.<Number>(BUFFER_SIZE, true);
			
			SOUND.addEventListener(SampleDataEvent.SAMPLE_DATA, handleSampleData);
			
			//-- delay 500ms to avoid click noise at startup
			setTimeout(start, 500);
		}

		/**
		 * @return : latency in milliSeconds
		 */
		public function getLatency() : Number
		{
			return mSoundLatency;
		}

		/**
		 * 
		 */
		private function start() : void
		{
			mLastBuffer = getTimer();
			
			mSoundChannel = SOUND.play();
		}

		/**
		 * 
		 */
		private function handleSampleData(inEvent : SampleDataEvent) : void
		{
			// Latency in milliSeconds. This can vary and needs to be calculated each SampleDataEvent.
			// The formula is ((SampleDataEvent.position / 44.1) - SoundChannelObject.position)
			if(mSoundChannel) mSoundLatency = (inEvent.position * 2.267573696145e-02) - mSoundChannel.position;
			
			clearBuffer();
			
			mEngine.process(BUFFER);

			var l : Vector.<Number> = BUFFER[0]; // left channel
			var r : Vector.<Number> = BUFFER[1]; // right channel
			var i : int = 0;
			var n : int = BUFFER_SIZE;
			
			for(;i < n; ++i)
			{
				inEvent.data.writeFloat(l[i]);
				inEvent.data.writeFloat(r[i]);
			}
		}

		/**
		 * 
		 */
		private function clearBuffer() : void
		{
			var l : Vector.<Number> = BUFFER[0];
			var r : Vector.<Number> = BUFFER[1];
			
			for(var i : int = 0 ;i < BUFFER_SIZE; ++i)
			{
				l[i] = 0.0;
				r[i] = 0.0;
			}
		}
	}
}
